import json

import requests


def get_area_lat_lon(area_name: str) -> tuple[float, float]:
    """Get the latitude and longitude of an area from Nominatim.

    Uses the [Nominatim API](https://nominatim.org/release-docs/develop/api/Search/).

    Args:
        area_name: The name of the area.

    Returns:
        The area found.

    """
    response = requests.get(
        f"https://nominatim.openstreetmap.org/search?q={area_name}&format=jsonv2",
        headers={"User-Agent": "Mozilla/5.0"},
    )
    response.raise_for_status()
    area = json.loads(response.content.decode())
    return area[0]["lat"], area[0]["lon"]


def driving_hours_to_meters(driving_hours: int) -> int:
    """Convert driving hours to meters assuming a 70 km/h average speed.

    Args:
        driving_hours: The driving hours.

    Returns:
        The distance in meters.

    """
    return driving_hours * 70 * 1000


def get_lat_lon_center(bounds: dict) -> tuple[float, float]:
    """Get the latitude and longitude of the center of a bounding box.

    Args:
        bounds: The bounding box.

            ```json
            {
                "minlat": float,
                "minlon": float,
                "maxlat": float,
                "maxlon": float,
            }
            ```

    Returns:
        The latitude and longitude of the center.

    """
    return (
        (bounds["minlat"] + bounds["maxlat"]) / 2,
        (bounds["minlon"] + bounds["maxlon"]) / 2,
    )


def get_surfing_spots(
    lat: float, lon: float, radius: int
) -> list[tuple[str, tuple[float, float]]]:
    """Get surfing spots around a given latitude and longitude.

    Uses the [Overpass API](https://wiki.openstreetmap.org/wiki/Overpass_API).

    Args:
        lat: The latitude.
        lon: The longitude.
        radius: The radius in meters.

    Returns:
        The surfing places found.

    """
    overpass_url = "https://overpass-api.de/api/interpreter"
    query = "[out:json];("
    query += f'nwr["natural"="beach"](around:{radius},{lat},{lon});'
    query += f'nwr["natural"="reef"](around:{radius},{lat},{lon});'
    query += ");out body geom;"
    params = {"data": query}
    response = requests.get(
        overpass_url, params=params, headers={"User-Agent": "Mozilla/5.0"}
    )
    response.raise_for_status()
    elements = response.json()["elements"]
    return [
        (element.get("tags", {}).get("name", ""), get_lat_lon_center(element["bounds"]))
        for element in elements
        if "surfing" in element.get("tags", {}).get("sport", "") and "bounds" in element
    ]



import os

from tools import (
    get_area_lat_lon,
    get_wave_forecast,
    get_wind_forecast,
)

from any_agent.evaluation import EvaluationCase
from any_agent.logging import logger
from any_agent.tools.web_browsing import search_tavily, search_web, visit_webpage

MODEL_OPTIONS = [
    # "huggingface/novita/deepseek-ai/DeepSeek-V3",
    # "huggingface/novita/meta-llama/Llama-3.3-70B-Instruct",
    "openai/gpt-4.1-nano",
    "openai/gpt-4.1-mini",
    "openai/gpt-4o",
    "gemini/gemini-2.0-flash-lite",
    "gemini/gemini-2.0-flash",
    # "huggingface/Qwen/Qwen3-32B", # right now throwing an internal error, but novita qwen isn't supporting tool calling
]

# Novita was the only HF based provider that worked.

# Hugginface API Provider Error:
# Must alternate between assistant/user, which meant that the 'tool' role made it puke


DEFAULT_EVALUATION_CASE = EvaluationCase(
    llm_judge=MODEL_OPTIONS[0],
    checkpoints=[
        {
            "criteria": "Check if the agent considered at least three surf spot options",
            "points": 1,
        },
        {
            "criteria": "Check if the agent gathered wind forecasts for each surf spot being evaluated.",
            "points": 1,
        },
        {
            "criteria": "Check if the agent gathered wave forecasts for each surf spot being evaluated.",
            "points": 1,
        },
        {
            "criteria": "Check if the agent used any web search tools to explore which surf spots should be considered",
            "points": 1,
        },
        {
            "criteria": "Check if the final answer contains any description about the weather (air temp, chance of rain, etc) at the chosen location",
            "points": 1,
        },
        {
            "criteria": "Check if the final answer includes one of the surf spots evaluated by tools",
            "points": 1,
        },
        {
            "criteria": "Check if the final answer includes information about some alternative surf spots if the user is not satisfied with the chosen one",
            "points": 1,
        },
    ],
)


DEFAULT_TOOLS = [
    get_wind_forecast,
    get_wave_forecast,
    get_area_lat_lon,
    search_web,
    visit_webpage,
]
if os.getenv("TAVILY_API_KEY"):
    DEFAULT_TOOLS.append(search_tavily)
else:
    logger.warning("TAVILY_API_KEY not set, skipping Tavily search tool")



from collections.abc import Callable, Sequence
from typing import TYPE_CHECKING

from opentelemetry.sdk.trace.export import (
    SimpleSpanProcessor,
    SpanExporter,
    SpanExportResult,
)

from any_agent import AgentFramework, AnyAgent
from any_agent.tracing import TracingProcessor
from any_agent.tracing.trace import AgentSpan

if TYPE_CHECKING:
    from opentelemetry.sdk.trace import ReadableSpan


class StreamlitExporter(SpanExporter):
    """Build an `AgentTrace` and export to the different outputs."""

    def __init__(  # noqa: D107
        self, agent_framework: AgentFramework, callback: Callable
    ):
        self.agent_framework = agent_framework
        self.processor: TracingProcessor | None = TracingProcessor.create(
            agent_framework
        )
        self.callback = callback

    def export(self, spans: Sequence["ReadableSpan"]) -> SpanExportResult:  # noqa: D102
        if not self.processor:
            return SpanExportResult.SUCCESS

        for readable_span in spans:
            # Check if this span belongs to our run
            span = AgentSpan.from_readable_span(readable_span)
            self.callback(span)

        return SpanExportResult.SUCCESS


def export_logs(agent: AnyAgent, callback: Callable) -> None:
    exporter = StreamlitExporter(agent.framework, callback)
    span_processor = SimpleSpanProcessor(exporter)
    agent._tracer_provider.add_span_processor(span_processor)



import json

import streamlit as st
from components.agent_status import export_logs
from components.inputs import UserInputs
from constants import DEFAULT_TOOLS
from surf_spot_finder.config import Config

from any_agent import AgentConfig, AgentFramework, AnyAgent, TracingConfig
from any_agent.evaluation import TraceEvaluationResult, evaluate
from any_agent.tracing.otel_types import StatusCode
from any_agent.tracing.trace import AgentSpan, AgentTrace


async def display_evaluation_results(result: TraceEvaluationResult):
    if result.ground_truth_result is not None:
        all_results = [*result.checkpoint_results, result.ground_truth_result]
    else:
        all_results = result.checkpoint_results

    # Create columns for better layout
    col1, col2 = st.columns(2)

    with col1:
        st.markdown("#### Criteria Results")
        for checkpoint in all_results:
            if checkpoint.passed:
                st.success(f"✅ {checkpoint.criteria}")
            else:
                st.error(f"❌ {checkpoint.criteria}")

    with col2:
        st.markdown("#### Overall Score")
        total_points = sum([result.points for result in all_results])
        if total_points == 0:
            msg = "Total points is 0, cannot calculate score."
            raise ValueError(msg)
        passed_points = sum([result.points for result in all_results if result.passed])

        # Create a nice score display
        st.markdown(f"### {passed_points}/{total_points}")
        percentage = (passed_points / total_points) * 100
        st.progress(percentage / 100)
        st.markdown(f"**{percentage:.1f}%**")


async def evaluate_agent(
    config: Config, agent_trace: AgentTrace
) -> TraceEvaluationResult:
    assert len(config.evaluation_cases) == 1, (
        "Only one evaluation case is supported in the demo"
    )
    st.markdown("### 📊 Evaluation Results")

    with st.spinner("Evaluating results..."):
        case = config.evaluation_cases[0]
        result: TraceEvaluationResult = evaluate(
            evaluation_case=case,
            trace=agent_trace,
            agent_framework=config.framework,
        )
    return result


async def configure_agent(user_inputs: UserInputs) -> tuple[AnyAgent, Config]:
    if "huggingface" in user_inputs.model_id:
        model_args = {
            "extra_headers": {"X-HF-Bill-To": "mozilla-ai"},
            "temperature": 0.0,
        }
    else:
        model_args = {}

    if user_inputs.framework == AgentFramework.AGNO:
        agent_args = {"tool_call_limit": 20}
    else:
        agent_args = {}

    agent_config = AgentConfig(
        model_id=user_inputs.model_id,
        model_args=model_args,
        agent_args=agent_args,
        tools=DEFAULT_TOOLS,
    )

    config = Config(
        location=user_inputs.location,
        max_driving_hours=user_inputs.max_driving_hours,
        date=user_inputs.date,
        framework=user_inputs.framework,
        main_agent=agent_config,
        managed_agents=[],
        evaluation_cases=[user_inputs.evaluation_case],
    )

    agent = await AnyAgent.create_async(
        agent_framework=config.framework,
        agent_config=config.main_agent,
        managed_agents=config.managed_agents,
        tracing=TracingConfig(console=True, cost_info=True),
    )
    return agent, config


async def display_output(agent_trace: AgentTrace):
    # Display the agent trace in a more organized way
    with st.expander("### 🧩 Agent Trace"):
        for span in agent_trace.spans:
            # Header with name and status
            col1, col2 = st.columns([4, 1])
            with col1:
                st.markdown(f"**{span.name}**")
                if span.attributes:
                    # st.json(span.attributes, expanded=False)
                    if "input.value" in span.attributes:
                        try:
                            input_value = json.loads(span.attributes["input.value"])
                            if isinstance(input_value, list) and len(input_value) > 0:
                                st.write(f"Input: {input_value[-1]}")
                            else:
                                st.write(f"Input: {input_value}")
                        except Exception:
                            st.write(f"Input: {span.attributes['input.value']}")
                    if "output.value" in span.attributes:
                        try:
                            output_value = json.loads(span.attributes["output.value"])
                            if isinstance(output_value, list) and len(output_value) > 0:
                                st.write(f"Output: {output_value[-1]}")
                            else:
                                st.write(f"Output: {output_value}")
                        except Exception:
                            st.write(f"Output: {span.attributes['output.value']}")
            with col2:
                status_color = (
                    "green" if span.status.status_code == StatusCode.OK else "red"
                )
                st.markdown(
                    f"<span style='color: {status_color}'>● {span.status.status_code.name}</span>",
                    unsafe_allow_html=True,
                )

    with st.expander("### 🏄 Results", expanded=True):
        time_col, cost_col, tokens_col = st.columns(3)
        duration = agent_trace.duration.total_seconds()
        with time_col:
            st.info(f"⏱️ Execution Time: {duration:0.2f} seconds")
        with cost_col:
            st.info(f"💰 Estimated Cost: ${agent_trace.cost.total_cost:.6f}")
        with tokens_col:
            st.info(f"📦 Total Tokens: {agent_trace.usage.total_tokens:,}")
        st.markdown("#### Final Output")
        st.info(agent_trace.final_output)


async def run_agent(agent, config) -> AgentTrace:
    st.markdown("#### 🔍 Running Surf Spot Finder with query")

    query = config.input_prompt_template.format(
        LOCATION=config.location,
        MAX_DRIVING_HOURS=config.max_driving_hours,
        DATE=config.date,
    )

    st.code(query, language="text")
    kwargs = {}
    if (
        config.framework == AgentFramework.OPENAI
        or config.framework == AgentFramework.TINYAGENT
    ):
        kwargs["max_turns"] = 20
    elif config.framework == AgentFramework.SMOLAGENTS:
        kwargs["max_steps"] = 20
    if config.framework == AgentFramework.LANGCHAIN:
        from langchain_core.runnables import RunnableConfig

        kwargs["config"] = RunnableConfig(recursion_limit=20)
    elif config.framework == AgentFramework.GOOGLE:
        from google.adk.agents.run_config import RunConfig

        kwargs["run_config"] = RunConfig(max_llm_calls=20)

    with st.status("Agent is running...", expanded=False, state="running") as status:

        def update_span(span: AgentSpan):
            # Process input value
            input_value = span.attributes.get("input.value", "")
            if input_value:
                try:
                    parsed_input = json.loads(input_value)
                    if isinstance(parsed_input, list) and len(parsed_input) > 0:
                        input_value = str(parsed_input[-1])
                except Exception:
                    pass

            # Process output value
            output_value = span.attributes.get("output.value", "")
            if output_value:
                try:
                    parsed_output = json.loads(output_value)
                    if isinstance(parsed_output, list) and len(parsed_output) > 0:
                        output_value = str(parsed_output[-1])
                except Exception:
                    pass

            # Truncate long values
            max_length = 800
            if len(input_value) > max_length:
                input_value = f"[Truncated]...{input_value[-max_length:]}"
            if len(output_value) > max_length:
                output_value = f"[Truncated]...{output_value[-max_length:]}"

            # Create a cleaner message format
            if input_value or output_value:
                message = f"Step: {span.name}\n"
                if input_value:
                    message += f"Input: {input_value}\n"
                if output_value:
                    message += f"Output: {output_value}"
            else:
                message = f"Step: {span.name}\n{span}"

            status.update(label=message, expanded=False, state="running")

        export_logs(agent, update_span)
        agent_trace: AgentTrace = await agent.run_async(query, **kwargs)
        status.update(label="Finished!", expanded=False, state="complete")

        agent.exit()
        return agent_trace



import os
import tempfile
from datetime import datetime, timedelta
from typing import Annotated

import geocoder
import yaml
from litellm.litellm_core_utils.get_llm_provider_logic import (
    get_llm_provider,
)
from pydantic import AfterValidator, BaseModel, ConfigDict, FutureDatetime, PositiveInt
from rich.prompt import Prompt

from any_agent import AgentFramework
from any_agent.config import AgentConfig
from any_agent.evaluation import EvaluationCase
from any_agent.logging import logger

INPUT_PROMPT_TEMPLATE = """
According to the forecast, what will be the best spot to surf around {LOCATION},
in a {MAX_DRIVING_HOURS} hour driving radius,
at {DATE}?"
""".strip()


def validate_prompt(value) -> str:
    for placeholder in ("{LOCATION}", "{MAX_DRIVING_HOURS}", "{DATE}"):
        if placeholder not in value:
            raise ValueError(f"prompt must contain {placeholder}")
    return value


def ask_framework() -> AgentFramework:
    """Ask the user which framework they would like to use. They must select one of the Agent Frameworks"""
    frameworks = [framework.name for framework in AgentFramework]
    frameworks_str = "\n".join(
        [f"{i}: {framework}" for i, framework in enumerate(frameworks)]
    )
    prompt = f"Select the agent framework to use:\n{frameworks_str}\n"
    choice = Prompt.ask(prompt, default="0")
    try:
        choice = int(choice)
        if choice < 0 or choice >= len(frameworks):
            raise ValueError("Invalid choice")
        return AgentFramework[frameworks[choice]]
    except ValueError:
        raise ValueError("Invalid choice")


def date_picker() -> FutureDatetime:
    """Ask the user to select a date in the future. The date must be at least 1 day in the future."""
    prompt = "Select a date in the future (YYYY-MM-DD-HH)"
    # the default should be the current date + 1 day
    now = datetime.now()
    default_val = (now + timedelta(days=1)).strftime("%Y-%m-%d-%H")
    date_str = Prompt.ask(prompt, default=default_val)
    try:
        year, month, day, hour = map(int, date_str.split("-"))
        date = datetime(year, month, day, hour)
        return date
    except ValueError:
        raise ValueError("Invalid date format. Please use YYYY-MM-DD-HH.")


def location_picker() -> str:
    """Ask the user to input a location. By default use the current location based on the IP address."""
    prompt = "Enter a location"
    g = geocoder.ip("me")
    default_val = f"{g.city} {g.state}, {g.country}"
    location = Prompt.ask(prompt, default=default_val)
    if not location:
        raise ValueError("location cannot be empty")
    return location


def max_driving_hours_picker() -> int:
    """Ask the user to input the maximum driving hours. The default is 2 hours."""
    prompt = "Enter the maximum driving hours"
    default_val = str(2)
    max_driving_hours = Prompt.ask(prompt, default=default_val)
    try:
        max_driving_hours = int(max_driving_hours)
        if max_driving_hours <= 0:
            raise ValueError("Invalid choice")
        return max_driving_hours
    except ValueError:
        raise ValueError("Invalid choice")


def get_litellm_model_id(agent_name) -> str:
    """Ask the user to input a model_id string. Validate it using the litellm.validate_environment function"""
    from litellm.utils import validate_environment

    prompt = f"Enter a valid model_id for agent {agent_name} using LiteLLM syntax"
    default_val = "openai/gpt-4o"
    model_id = Prompt.ask(prompt, default=default_val)
    # make a call to validate the model id: this will throw an error if the model id is not valid
    get_llm_provider(model=model_id)
    # make a call to validate that the environment is correct for the model id
    env_check = validate_environment(model_id)
    if not env_check["keys_in_environment"]:
        msg = f"{env_check['missing_keys']} needed for {model_id}"
        raise ValueError(msg)
    return model_id


def set_mcp_settings(tool):
    logger.info(
        f"This MCP uses {tool['command']}. If you don't have this set up this will not work"
    )
    if "mcp/filesystem" not in tool["args"]:
        msg = "The only MCP that this demo supports is the filesystem MCP"
        raise ValueError(msg)
    if not any("{{ path_variable }}" in arg for arg in tool["args"]):
        msg = "The filesystem MCP must have { path_variable } in the args list"
        raise ValueError(msg)
    for idx, item in enumerate(tool["args"]):
        if "{{ path_variable }}" in item:
            default_val = os.path.join(tempfile.gettempdir(), "surf_spot_finder")
            answer = Prompt.ask(
                "Please enter the path you'd like the Filesystem MCP to access",
                default=default_val,
            )
            os.makedirs(answer, exist_ok=True)
            tool["args"][idx] = item.replace("{{ path_variable }}", answer)
    return tool


class Config(BaseModel):
    model_config = ConfigDict(extra="forbid")

    location: str
    max_driving_hours: PositiveInt
    date: FutureDatetime
    input_prompt_template: Annotated[str, AfterValidator(validate_prompt)] = (
        INPUT_PROMPT_TEMPLATE
    )

    framework: AgentFramework

    main_agent: AgentConfig
    managed_agents: list[AgentConfig] | None = None

    evaluation_cases: list[EvaluationCase] | None = None

    @classmethod
    def from_dict(cls, data: dict) -> "Config":
        """Create a Config instance from a dictionary.

        Args:
            data (dict): A dictionary containing the configuration data.

        Returns:
            Config: A new Config instance populated with values from the dictionary.

        """
        # for each tool listed in main_agent.tools, use import lib to import it and replace the str with the callable
        callables = []
        if data.get("main_agent") is None:
            data["main_agent"] = {}
        if not data["main_agent"].get("model_id"):
            data["main_agent"]["model_id"] = get_litellm_model_id("main_agent")
        else:
            logger.info(f"Main agent using model_id {data['main_agent']['model_id']}")
        for tool in data["main_agent"].get("tools", []):
            if isinstance(tool, str):
                module_name, func_name = tool.rsplit(".", 1)
                module = __import__(module_name, fromlist=[func_name])
                callables.append(getattr(module, func_name))
            else:
                # this means it must be an MCPStdioParams
                # For the purposes of this demo, currently we just look for the filesystem MCP which we have a placeholder
                # for the path variable (which controls which dirs the MCP will have access to).
                mcp_tool = set_mcp_settings(tool)
                callables.append(mcp_tool)
        data["main_agent"]["tools"] = callables
        for agent in data.get("managed_agents", []):
            if agent.get("model_id") is None:
                agent["model_id"] = get_litellm_model_id(
                    agent.get("name", "managed_agent")
                )
            else:
                logger.info(f"Agent {agent['name']} using model_id {agent['model_id']}")
            callables = []
            for tool in agent.get("tools", []):
                if isinstance(tool, str):
                    module_name, func_name = tool.rsplit(".", 1)
                    module = __import__(module_name, fromlist=[func_name])
                    callables.append(getattr(module, func_name))
                else:
                    # this means it must be an MCPStdioParams
                    mcp_tool = set_mcp_settings(tool)
                    callables.append(mcp_tool)
            agent["tools"] = callables
        if not data.get("framework"):
            data["framework"] = ask_framework()
        else:
            logger.info(f"Using framework {data['framework']}")
        if not data.get("location"):
            data["location"] = location_picker()
        else:
            logger.info(f"Using location {data['location']}")
        if not data.get("max_driving_hours"):
            data["max_driving_hours"] = max_driving_hours_picker()
        else:
            logger.info(f"Using max driving hours {data['max_driving_hours']}")
        if not data.get("date"):
            data["date"] = date_picker()
        else:
            logger.info(f"Using date {data['date']}")

        return cls(**data)

    @classmethod
    def from_yaml(cls, yaml_path: str) -> "Config":
        """With open(yaml_path, "r") as f:
            data = yaml.safe_load(f)
        return cls(**data)    yaml_path: Path to the YAML configuration file

        Returns:
            Config: A new Config instance populated with values from the YAML file

        """
        with open(yaml_path) as f:
            data = yaml.safe_load(f)
        return cls.from_dict(data)
